package org.eco.als.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.eco.als.domain.bo.ModelBo;
import org.eco.als.domain.vo.ModelImportVo;
import org.eco.als.domain.vo.ModelVo;
import org.eco.als.service.IModelService;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.excel.utils.ExcelUtil;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.web.core.BaseController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;

/**
 * 模型信息Controller
 *
 * @author wgk
 * @date 2024-07-22
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/model")
public class ModelController extends BaseController {
    @Resource
    private IModelService modelService;

    /**
     * 查询模型信息列表
     */
    @SaCheckPermission("als:model:list")
    @GetMapping("/list")
    public CommonResult<PageResult<ModelVo>> list(ModelBo modelBo) {
        return CommonResult.success(modelService.selectPage(modelBo));
    }

    /**
     * 导出模型信息列表
     */
    @SaCheckPermission("als:model:export")
    @Log(title = "模型信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(HttpServletResponse response, ModelBo modelBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<ModelVo> list = modelService.selectList(modelBo);
        modelService.asyncExport(list, "模型信息", loginUser);
        return CommonResult.success();
    }

    @SaCheckPermission("als:model:import")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "模型信息", ModelImportVo.class, response);
    }

    /**
     * 导入模型信息列表
     */
    @Log(title = "模型信息", businessType = BusinessType.IMPORT)
    @SaCheckPermission("als:model:import")
    @PostMapping("/importData")
    public CommonResult<Void> importData(MultipartFile file, boolean updateSupport) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        modelService.asyncImportData(file, updateSupport, loginUser);
        return CommonResult.success();
    }

    /**
     * 获取模型信息详细信息
     */
    @SaCheckPermission("als:model:query")
    @GetMapping(value = "/{id}")
    public CommonResult<ModelVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(modelService.selectById(id));
    }

    /**
     * 新增模型信息
     */
    @SaCheckPermission("als:model:add")
    @Log(title = "模型信息", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody ModelBo modelBo) {
        boolean inserted = modelService.insert(modelBo);
        if (!inserted) {
            return CommonResult.fail("新增模型信息记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改模型信息
     */
    @SaCheckPermission("als:model:edit")
    @Log(title = "模型信息", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody ModelBo modelBo) {
        boolean updated = modelService.update(modelBo);
        if (!updated) {
            return CommonResult.fail("修改模型信息记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除模型信息
     */
    @SaCheckPermission("als:model:remove")
    @Log(title = "模型信息", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = modelService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除模型信息记录失败!");
        }
        return CommonResult.success();
    }
}
