package org.eco.als.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.collection.CollUtil;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.eco.als.domain.bo.SafeRuleBo;
import org.eco.als.domain.vo.SafeRuleImportVo;
import org.eco.als.domain.vo.SafeRuleVo;
import org.eco.als.service.ISafeRuleService;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.excel.utils.ExcelUtil;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.web.core.BaseController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;

/**
 * 保障规则Controller
 *
 * @author wgk
 * @date 2024-07-24
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/safeRule")
public class SafeRuleController extends BaseController {
    @Resource
    private ISafeRuleService safeRuleService;

    /**
     * 查询保障规则列表
     */
    @SaCheckPermission("als:safeRule:list")
    @GetMapping("/list")
    public CommonResult<PageResult<SafeRuleVo>> list(SafeRuleBo safeRuleBo) {
        return CommonResult.success(safeRuleService.selectPage(safeRuleBo));
    }

    /**
     * 导出保障规则列表
     */
    @SaCheckPermission("als:safeRule:export")
    @Log(title = "保障规则", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(SafeRuleBo safeRuleBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<SafeRuleVo> list = safeRuleService.selectList(safeRuleBo);
        if (CollUtil.isEmpty(list)) {
            return CommonResult.fail("导出列表为空");
        }
        safeRuleService.asyncExport(list, "保障规则", loginUser);
        return CommonResult.success();
    }

    @SaCheckPermission("als:safeRule:import")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "保障规则", SafeRuleImportVo.class, response);
    }

    /**
     * 导入保障规则列表
     */
    @Log(title = "保障规则", businessType = BusinessType.IMPORT)
    @SaCheckPermission("als:safeRule:import")
    @PostMapping("/importData")
    public CommonResult<Void> importData(MultipartFile file, boolean updateSupport) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        safeRuleService.asyncImportData(file, updateSupport, loginUser);
        return CommonResult.success();
    }

    /**
     * 获取保障规则详细信息
     */
    @SaCheckPermission("als:safeRule:query")
    @GetMapping(value = "/{id}")
    public CommonResult<SafeRuleVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(safeRuleService.selectById(id));
    }

    /**
     * 新增保障规则
     */
    @SaCheckPermission("als:safeRule:add")
    @Log(title = "保障规则", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody SafeRuleBo safeRuleBo) {
        boolean inserted = safeRuleService.insert(safeRuleBo);
        if (!inserted) {
            return CommonResult.fail("新增保障规则记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改保障规则
     */
    @SaCheckPermission("als:safeRule:edit")
    @Log(title = "保障规则", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody SafeRuleBo safeRuleBo) {
        boolean updated = safeRuleService.update(safeRuleBo);
        if (!updated) {
            return CommonResult.fail("修改保障规则记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除保障规则
     */
    @SaCheckPermission("als:safeRule:remove")
    @Log(title = "保障规则", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = safeRuleService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除保障规则记录失败!");
        }
        return CommonResult.success();
    }
}
